import numpy as np

class Mat3:

    def __init__(self, top, middle, bottom):
        if top is None:
            top = (0, 0, 0)
        if middle is None:
            middle = (0, 0, 0)
        if bottom is None:
            bottom = (0, 0, 0)
        self.x00 = top[0]
        self.x01 = top[1]
        self.x02 = top[2]
        self.x10 = middle[0]
        self.x11 = middle[1]
        self.x12 = middle[2]
        self.x20 = bottom[0]
        self.x21 = bottom[1]
        self.x22 = bottom[2]

    @classmethod
    def identity(cls):
        top = (1, 0, 0)
        middle = (0, 1, 0)
        bottom = (0, 0, 1)
        return cls(top, middle, bottom)

    @property
    def Identity(self):
        return Mat3((1, 0, 0), (0, 1, 0), (0, 0, 1))

    @property
    def determ(self):
        return (self.x00 * ((self.x11 * self.x22) - (self.x12 * self.x21))) - (self.x01 * ((self.x10 * self.x22) - (self.x12 * self.x20))) + (self.x02 * ((self.x10 * self.x21) - (self.x11 * self.x20)))

    @property
    def transpose(self):
        top = (self.x00, self.x10, self.x20)
        middle = (self.x01, self.x11, self.x21)
        bottom = (self.x02, self.x12, self.x22) 
        return Mat3(top, middle, bottom)

    @property
    def inverse(self):
        a = np.array([[self.x00, self.x01, self.x02], [self.x10, self.x11, self.x12], [self.x20, self.x21, self.x22]])
        b = np.linalg.inv(a)
        return Mat3(b[0], b[1], b[2])

    def multM3(self, other):
        M = other
        top0 = self.x00 * M.x00 + self.x01 * M.x10 + self.x02 * M.x20
        top1 = self.x00 * M.x01 + self.x01 * M.x11 + self.x02 * M.x21
        top2 = self.x00 * M.x02 + self.x01 * M.x12 + self.x02 * M.x22
        middle0 = self.x10 * M.x00 + self.x11 * M.x10 + self.x12 * M.x20
        middle1 = self.x10 * M.x01 + self.x11 * M.x11 + self.x12 * M.x21
        middle2 = self.x10 * M.x02 + self.x11 * M.x12 + self.x12 * M.x22
        bottom0 = self.x20 * M.x00 + self.x21 * M.x10 + self.x22 * M.x20
        bottom1 = self.x20 * M.x01 + self.x21 * M.x11 + self.x22 * M.x21
        bottom2 = self.x20 * M.x02 + self.x21 * M.x12 + self.x22 * M.x22
        top = (top0, top1, top2)
        middle = (middle0, middle1, middle2)
        bottom = (bottom0, bottom1, bottom2)
        return Mat3(top, middle, bottom)

    def __add__(self, other):
        t = (self.x00 + other.x00, self.x01 + other.x01, self.x02 + other.x02)
        m = (self.x10 + other.x10, self.x11 + other.x11, self.x12 + other.x12)
        b = (self.x20 + other.x20, self.x21 + other.x21, self.x22 + other.x22)
        return Mat3(t, m, b)

    def __sub__(self, other):
        t = (self.x00 - other.x00, self.x01 - other.x01, self.x02 - other.x02)
        m = (self.x10 - other.x10, self.x11 - other.x11, self.x12 - other.x12)
        b = (self.x20 - other.x20, self.x21 - other.x21, self.x22 - other.x22)
        return Mat3(t, m, b)

    def __mul__(self, other):
        top = (self.x00 * other, self.x01 * other, self.x02 * other)
        middle = (self.x10 * other, self.x11 * other, self.x12 * other)
        bottom = (self.x20 * other, self.x21 * other, self.x22 * other)
        return Mat3(top, middle, bottom)

    def __rmul__(self, other):
        top = (self.x00 * other, self.x01 * other, self.x02 * other)
        middle = (self.x10 * other, self.x11 * other, self.x12 * other)
        bottom = (self.x20 * other, self.x21 * other, self.x22 * other)
        return Mat3(top, middle, bottom)

    def __repr__(self):
        return str([[self.x00, self.x01, self.x02], [self.x10, self.x11, self.x12], [self.x20, self.x21, self.x22]])

    def __str__(self):
        return 'Mat3: ' + str([[self.x00, self.x01, self.x02], [self.x10, self.x11, self.x12], [self.x20, self.x21, self.x22]])