#import os
#os.chdir(os.path.join('C:\\', 'Users', 'gb2', 'Documents', 'PhD', 'Python'))
import math
from vec3 import Vec3
from frame import Frame


class Pose:
    def __init__(self, camera):
        self._frame = Frame.universal()
        self._parentFrame = Frame.universal()
        self._camera = camera
        camera.frame = self._frame
        self._azimuth = 0
        self._polar = math.pi * 0.5
        self._roll = 0

    def _setOrientation(self, azimuth, polar, roll):
        v = self._parentFrame.v.rotate(self._parentFrame.w, azimuth)
        u = self._parentFrame.w.rotate(v, polar)
        v = v.rotate(-1 * u, roll)
        w = u.cross(v)
        self._frame.axes = (u, v, w)
        cameraFrame = Frame(self._frame.v.negated, self._frame.w.negated, self._frame.u, self._frame.origin)
        self._camera.frame = cameraFrame

    @property
    def azimuth(self):
        return self._azimuth

    @azimuth.setter
    def azimuth(self, newAzimuth):
        self._azimuth = newAzimuth
        self._setOrientation(newAzimuth, self._polar, self._roll)

    @property
    def polar(self):
        return self._polar

    @polar.setter
    def polar(self, newPolar):
        self._polar = newPolar
        self._setOrientation(self._azimuth, newPolar, self._roll)

    @property
    def roll(self):
        return self._roll

    @roll.setter
    def roll(self, newRoll):
        self._roll = newRoll
        self._setOrientation(self._azimuth, self._polar, newRoll)

    @property
    def position(self):
        return self._frame._origin

    @position.setter
    def position(self, newPosition):
        self._frame.origin = newPosition
        self._camera.position = newPosition

    @property
    def frame(self):
        return self._frame

    @property
    def parentFrame(self):
        return self._parentFrame

    @parentFrame.setter
    def parentFrame(self, newParentFrame):
        self._parentFrame = newParentFrame
        self.position = newParentFrame.origin
        self._setOrientation(self._azimuth, self._polar, self._roll)

    @property
    def camera(self):
        return self._camera