import numpy as np
import matplotlib as mpl
from matplotlib import pyplot as plt
import matplotlib.cm as cm
import matplotlib.colors as mcolors
from matplotlib.colors import ListedColormap

seq = mpl.cm.get_cmap('viridis')
seq2 = mpl.cm.get_cmap('magma')
seq3 = mpl.cm.get_cmap('autumn')
grey = mpl.cm.get_cmap('gray')
div = mpl.cm.get_cmap('PiYG')
div2 = mpl.cm.get_cmap('coolwarm')
cyc = mpl.cm.get_cmap('twilight')
filts = np.zeros((9, 4))
filts[0, :] = np.array([1, 1, 1, 1])
filts[1, :] = np.array([38/256, 22/256, 111/256, 1])
filts[2, :] = np.array([69/256, 109/256, 170/256, 1])
filts[3, :] = np.array([0/256, 146/256, 219/256, 1])
filts[4, :] = np.array([78/256, 205/256, 196/256, 1])
filts[5, :] = np.array([255/256, 230/256, 109/256, 1])
filts[6, :] = np.array([255/256, 107/256, 107/256, 1])
filts[7, :] = np.array([217/256, 35/256, 29/256, 1])
filts[8, :] = np.array([125/256, 20/256, 17/256, 1])
filts = ListedColormap(filts)

def colourHighlight(n, map=''):
    if map == 'div' or map == 'diverging':
        n = max(min(n, 3), 0)
        return ('#c31a7c', '#efb2d7', '#bce28d', '#519724')[n]
    elif map == 'seq2' or map == 'sequential2':
        n = max(min(n, 4), 0)
        return ('#3a0f6e', '#8a2881', '#da4769', '#fd9969', '#fcf3b5')[n]
    else:
        n = max(min(n, 4), 0)
        return ('#414286', '#2a768e', '#20a585', '#71ce56', '#f2e51d')[n]

def valueColour(value, map='', alpha=1.0):
    cmap = seq
    if map == 'grey' or map == 'greyscale' or map == 'gray':
        cmap = grey
    elif map == 'div' or map == 'diverging':
        cmap = div
    elif map == 'div2' or map == 'diverging2':
        cmap = div2
    elif map == 'cyc' or map == 'cyclic':
        cmap = cyc
    elif map == 'seq2' or map == 'sequential2':
        cmap = seq2
    elif map == 'seq3' or map == 'sequential3':
        cmap = seq3
    elif map == 'filts':
        cmap = filts
    return cmap(float(value))[:-1] + (alpha,)

def valueColours(values, map='', alpha=1.0):
    colours = []
    for value in values:
        colours += [valueColour(value, map=map, alpha=alpha)]
    return colours

def load(filename):
    path = '../data/products/{0}.npy'.format(filename)
    image = np.load(path)
    return image

def save(product, filename):
    path = '../data/products/{0}.npy'.format(filename)
    np.save(path, product)

def display(data, map='', vlims=(None, None), xtitle=None, ytitle=None, xticks=None, yticks=None, cb=0, cbticks=None,
            cbticklabs=None, cbtitle='', cbextend='neither', cbsegs=None, cbshrink=1, cbOrientation='vertical',
            interp=None, scaleFactor=1, saveWithName=None):
    if type(data) is str:
        data = load(data)
    elif type(data) is not np.ndarray:
        print('Cannot display data type {0}.'.format(type(data)))
    if map == 'grey' or map == 'greyscale' or map == 'gray':
        cmap = grey
    elif map == 'div' or map == 'diverging':
        cmap = div
    elif map == 'div2' or map == 'diverging2':
        cmap = div2
    elif map == 'cyc' or map == 'cyclic':
        cmap = cyc
    elif map == 'seq2' or map == 'sequential2':
        cmap = seq2
    elif map == 'seq3' or map == 'sequential3':
        cmap = seq3
    elif map == 'filts':
        cmap = filts
    else:
        cmap = seq
    if cbsegs:
        cmap = plt.cm.get_cmap('viridis', lut=8)
        #cmap = plt.cm.get_cmap('magma', lut=8)
    shape = data.shape
    imagePlot = plt.imshow(data, vmin=vlims[0], vmax=vlims[1], interpolation=interp)
    imagePlot.set_cmap(cmap)
    plt.tick_params(direction='inout', length=5)
    if xticks is not None:
        plt.xticks(np.linspace(0, shape[1], len(xticks)), xticks)
    if yticks is not None:
        plt.yticks(np.linspace(0, shape[0], len(yticks)), yticks)
    if xtitle is not None:
        plt.xlabel(xtitle)
    if ytitle is not None:
        plt.ylabel(ytitle)
    if cb:
        cbar = plt.colorbar(ticks=cbticks, extend=cbextend, shrink=cbshrink, orientation=cbOrientation)
        cbar.ax.tick_params(axis='y', direction='inout', length=5)
        cbar.set_label(cbtitle)
        if cbticklabs is not None:
            cbar.ax.set_yticklabels(cbticklabs)
    if scaleFactor != 1:
        figWidth, figHeight = plt.gcf().get_size_inches()
        plt.gcf().set_size_inches(figWidth * scaleFactor, figHeight * scaleFactor)
    if saveWithName is not None:
        path = '../data/products/figures/{0}.png'.format(saveWithName)
        plt.savefig(path, dpi=200)
    plt.show()

def addImageToPlot(plot, data, map='', vlims=(None, None), xtitle=None, ytitle=None, xticks=None, yticks=None, cb=0, interp=None,):
    if type(data) is str:
        product = load(data)
    elif type(data) is not np.ndarray:
        print('Cannot display data type {0}.'.format(type(data)))
    if map == 'grey' or map == 'greyscale' or map == 'gray':
        cmap = 'gray'
    elif map == 'div' or map == 'diverging':
        cmap = 'PiYG'
    elif map == 'cyc' or map == 'cyclic':
        cmap = 'twilight'
    elif map == 'seq2' or map == 'sequential2':
        cmap = 'magma'
    elif map == 'filts':
        cmap = filts
    else:
        cmap = 'viridis'
    shape = data.shape
    plot.imshow(data, vmin=vlims[0], vmax=vlims[1], interpolation=interp)
    plot.set_cmap(cmap)
    if xticks is not None:
        plot.xticks(np.linspace(0, shape[1], len(xticks)), xticks)
    if yticks is not None:
        plot.yticks(np.linspace(0, shape[0], len(yticks)), yticks)
    if xtitle is not None:
        plot.xlabel(xtitle)
    if ytitle is not None:
        plot.ylabel(ytitle)
    if cb:
        plot.colorbar()
    #plt.show()
